﻿using System.Text;

namespace anbs_cp.Extensions;

/// <summary>
/// Класс-расширение для List
/// </summary>
public static class ListExtensions
{
    /// <summary>
    /// Загрузка строк из текстового файла
    /// </summary>
    /// <param name="list">Список, в который нужно загрузит</param>
    /// <param name="fileName">Имя файла, который нужно загрузить</param>
    /// <param name="useUtf8WithBom">Использовать кодировку UTF8 With Bom</param>
    public static void LoadFromFile (this List<string> list, string fileName, bool useUtf8WithBom = false)
    {
        // Очищаю список
        list.Clear();

        // Кодировка
        Encoding fileEncoding =
            useUtf8WithBom ? Encoding.UTF8 : Encoding.UTF8.UTF8WithoutBom();

        // Открываю файл для чтения
        using StreamReader reader = new(new FileStream(fileName, FileMode.Open), fileEncoding);

        // Пока есть строки в файле
        while (reader.ReadLine() is { } line)
            // - добавляю в список
            list.Add(line);
    }

    /// <summary>
    /// Загрузка строк из типизированного файла
    /// </summary>
    /// <typeparam name="T">Тип данных</typeparam>
    /// <param name="list">Список, в который нужно загрузит</param>
    /// <param name="fileName">Имя файла, который нужно загрузить</param>
    /// <param name="encoding">Кодировка файла</param>
    public static void LoadFromFile<T> (this List<T> list, string fileName, Encoding encoding)
    {
        // Очищаю список
        list.Clear();

        // Добавляю в список загруженные строки
        list.AddRange(FileExtension.ReadAll<T>(fileName, encoding));
    }

    /// <summary>
    /// Сохранение списка строк в файл
    /// </summary>
    /// <param name="list">Список</param>
    /// <param name="fileName">Имя файла</param>
    /// <param name="useUtf8WithBom">Использовать кодировку UTF8 With Bom</param>
    public static void SaveToFile (this List<string> list, string fileName, bool useUtf8WithBom = false)
    {
        // Очищаю список
        list.Clear();

        // Кодировка
        Encoding fileEncoding =
            useUtf8WithBom ? Encoding.UTF8 : Encoding.UTF8.UTF8WithoutBom();

        // Открываю файл для создания или перезаписи
        using StreamWriter writer = new(new FileStream(fileName, FileMode.OpenOrCreate), fileEncoding);

        // Для каждой строки
        foreach (string line in list)
            // - записываю её
            writer.WriteLine(line);
    }

    /// <summary>
    /// Сохранение списка строк в файл
    /// </summary>
    /// <typeparam name="T">Тип данных</typeparam>
    /// <param name="list">Список</param>
    /// <param name="fileName">Имя файла</param>
    /// <param name="encoding">Кодировка файла</param>
    public static void SaveToFile<T> (this List<T> list, string fileName, Encoding encoding) =>
        FileExtension.WriteAll(list.AsEnumerable(), fileName, encoding);
}